﻿using System;
using System.Collections.Generic;
using Microsoft.EntityFrameworkCore;

namespace SocialEducatorAvalonia.Models;

public partial class SocialeducationContext : DbContext
{
    public SocialeducationContext()
    {
    }

    public SocialeducationContext(DbContextOptions<SocialeducationContext> options)
        : base(options)
    {
    }

    public virtual DbSet<Group> Groups { get; set; }

    public virtual DbSet<Student> Students { get; set; }

    public virtual DbSet<StudentsActiveAndGraduation> StudentsActiveAndGraduations { get; set; }

    public virtual DbSet<StudentsAndPayment> StudentsAndPayments { get; set; }
    public virtual DbSet<StudentsEnded> StudentsEndeds { get; set; }

    public virtual DbSet<StudentsLeaved> StudentsLeaveds { get; set; }

    public virtual DbSet<StudentsStatus> StudentsStatuses { get; set; }

    public virtual DbSet<TypesOfPayment> TypesOfPayments { get; set; }

    public virtual DbSet<ViewCountStudentsInGroup> ViewCountStudentsInGroups { get; set; }

    protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
#warning To protect potentially sensitive information in your connection string, you should move it out of source code. You can avoid scaffolding the connection string by using the Name= syntax to read it from configuration - see https://go.microsoft.com/fwlink/?linkid=2131148. For more guidance on storing connection strings, see https://go.microsoft.com/fwlink/?LinkId=723263.
        => optionsBuilder.UseNpgsql("Host=ngknn.ru;Port=5432;Username=SocialPedagog;Password=qQhReh;Database=SocialPedagogDB");

    protected override void OnModelCreating(ModelBuilder modelBuilder)
    {
        modelBuilder.Entity<Group>(entity =>
        {
            entity.HasKey(e => e.GroupId).HasName("groups_pk");

            entity.ToTable("groups");

            entity.HasIndex(e => e.Title, "groups_un").IsUnique();

            entity.Property(e => e.GroupId).HasColumnName("group_id");
            entity.Property(e => e.Title)
                .HasMaxLength(8)
                .HasColumnName("title");
        });

        modelBuilder.Entity<Student>(entity =>
        {
            entity.HasKey(e => e.StudentId).HasName("students_pk");

            entity.ToTable("students");

            entity.HasIndex(e => new { e.Fio, e.AdmissionDate, e.StudentStatusId, e.StudentGroupId }, "students_unique").IsUnique();

            entity.Property(e => e.StudentId).HasColumnName("student_id");
            entity.Property(e => e.AdmissionDate).HasColumnName("admission_date");
            entity.Property(e => e.Fio).HasColumnName("fio");
            entity.Property(e => e.Name)
                .HasMaxLength(64)
                .HasColumnName("name");
            entity.Property(e => e.Patronymic)
                .HasMaxLength(64)
                .HasColumnName("patronymic");
            entity.Property(e => e.StudentGroupId)
                .ValueGeneratedOnAdd()
                .HasColumnName("student_group_id");
            entity.Property(e => e.StudentStatusId)
                .ValueGeneratedOnAdd()
                .HasColumnName("student_status_id");
            entity.Property(e => e.Surname)
                .HasMaxLength(64)
                .HasColumnName("surname");

            entity.HasOne(d => d.StudentGroup).WithMany(p => p.Students)
                .HasForeignKey(d => d.StudentGroupId)
                .OnDelete(DeleteBehavior.ClientSetNull)
                .HasConstraintName("students_groups_fk");

            entity.HasOne(d => d.StudentStatus).WithMany(p => p.Students)
                .HasForeignKey(d => d.StudentStatusId)
                .OnDelete(DeleteBehavior.ClientSetNull)
                .HasConstraintName("students_students_status_fk");
        });

        modelBuilder.Entity<StudentsActiveAndGraduation>(entity =>
        {
            entity
                .HasNoKey()
                .ToTable("students_active_and_graduation");

            entity.Property(e => e.AdmissionDate).HasColumnName("admission_date");
            entity.Property(e => e.Fio).HasColumnName("fio");
            entity.Property(e => e.StudentGroupId)
                .HasDefaultValueSql("nextval('students_partitioned_student_group_id_seq'::regclass)")
                .HasColumnName("student_group_id");
            entity.Property(e => e.StudentId)
                .HasDefaultValueSql("nextval('students_partitioned_student_id_seq'::regclass)")
                .HasColumnName("student_id");
            entity.Property(e => e.StudentStatusId).HasColumnName("student_status_id");

            entity.HasOne(d => d.StudentGroup).WithMany()
                .HasForeignKey(d => d.StudentGroupId)
                .OnDelete(DeleteBehavior.ClientSetNull)
                .HasConstraintName("students_partitioned_groups_fk");

            entity.HasOne(d => d.StudentStatus).WithMany()
                .HasForeignKey(d => d.StudentStatusId)
                .OnDelete(DeleteBehavior.ClientSetNull)
                .HasConstraintName("students_partitioned_students_status_fk");
        });
        
        modelBuilder.Entity<StudentsEnded>(entity =>
        {
            entity
                .HasNoKey()
                .ToTable("students_ended");

            entity.Property(e => e.AdmissionDate).HasColumnName("admission_date");
            entity.Property(e => e.Fio).HasColumnName("fio");
            entity.Property(e => e.StudentGroupId)
                .HasDefaultValueSql("nextval('students_partitioned_student_group_id_seq'::regclass)")
                .HasColumnName("student_group_id");
            entity.Property(e => e.StudentId)
                .HasDefaultValueSql("nextval('students_partitioned_student_id_seq'::regclass)")
                .HasColumnName("student_id");
            entity.Property(e => e.StudentStatusId).HasColumnName("student_status_id");

            entity.HasOne(d => d.StudentGroup).WithMany()
                .HasForeignKey(d => d.StudentGroupId)
                .OnDelete(DeleteBehavior.ClientSetNull)
                .HasConstraintName("students_partitioned_groups_fk");

            entity.HasOne(d => d.StudentStatus).WithMany()
                .HasForeignKey(d => d.StudentStatusId)
                .OnDelete(DeleteBehavior.ClientSetNull)
                .HasConstraintName("students_partitioned_students_status_fk");
        });

        modelBuilder.Entity<StudentsAndPayment>(entity =>
        {
            entity.HasKey(e => e.StudentPaymentId).HasName("students_and_payments_pk");

            entity.ToTable("students_and_payments");

            entity.HasIndex(e => e.StudentPaymentId, "students_and_payments_unique").IsUnique();

            entity.Property(e => e.StudentPaymentId).HasColumnName("student_payment_id");
            entity.Property(e => e.AmountPayment)
                .HasColumnType("money")
                .HasColumnName("amount_payment");
            entity.Property(e => e.DataOfPayment).HasColumnName("data_of_payment");
            entity.Property(e => e.StudentId)
                .ValueGeneratedOnAdd()
                .HasColumnName("student_id");
            entity.Property(e => e.TypePaymentId)
                .ValueGeneratedOnAdd()
                .HasColumnName("type_payment_id");

            entity.HasOne(d => d.Student).WithMany(p => p.StudentsAndPayments)
                .HasForeignKey(d => d.StudentId)
                .OnDelete(DeleteBehavior.ClientSetNull)
                .HasConstraintName("students_and_payments_students_fk");

            entity.HasOne(d => d.TypePayment).WithMany(p => p.StudentsAndPayments)
                .HasForeignKey(d => d.TypePaymentId)
                .OnDelete(DeleteBehavior.ClientSetNull)
                .HasConstraintName("students_and_payments_types_of_payments_fk");
        });


        modelBuilder.Entity<StudentsLeaved>(entity =>
        {
            entity
                .HasNoKey()
                .ToTable("students_leaved");

            entity.Property(e => e.AdmissionDate).HasColumnName("admission_date");
            entity.Property(e => e.Fio).HasColumnName("fio");
            entity.Property(e => e.StudentGroupId)
                .HasDefaultValueSql("nextval('students_partitioned_student_group_id_seq'::regclass)")
                .HasColumnName("student_group_id");
            entity.Property(e => e.StudentId)
                .HasDefaultValueSql("nextval('students_partitioned_student_id_seq'::regclass)")
                .HasColumnName("student_id");
            entity.Property(e => e.StudentStatusId).HasColumnName("student_status_id");

            entity.HasOne(d => d.StudentGroup).WithMany()
                .HasForeignKey(d => d.StudentGroupId)
                .OnDelete(DeleteBehavior.ClientSetNull)
                .HasConstraintName("students_partitioned_groups_fk");

            entity.HasOne(d => d.StudentStatus).WithMany()
                .HasForeignKey(d => d.StudentStatusId)
                .OnDelete(DeleteBehavior.ClientSetNull)
                .HasConstraintName("students_partitioned_students_status_fk");
        });

        modelBuilder.Entity<StudentsStatus>(entity =>
        {
            entity.HasKey(e => e.StatusId).HasName("students_status_pk");

            entity.ToTable("students_status");

            entity.Property(e => e.StatusId).HasColumnName("status_id");
            entity.Property(e => e.Title)
                .HasMaxLength(40)
                .HasColumnName("title");
        });

        modelBuilder.Entity<TypesOfPayment>(entity =>
        {
            entity.HasKey(e => e.TypeOfPaymentId).HasName("types_of_payments_pk");

            entity.ToTable("types_of_payments");

            entity.HasIndex(e => e.TypeOfPaymentId, "types_of_payments_unique").IsUnique();

            entity.Property(e => e.TypeOfPaymentId).HasColumnName("type_of_payment_id");
            entity.Property(e => e.TypeTittle)
                .HasMaxLength(60)
                .HasColumnName("type_tittle");
        });

        modelBuilder.Entity<ViewCountStudentsInGroup>(entity =>
        {
            entity
                .HasNoKey()
                .ToView("view_count_students_in_group");

            entity.Property(e => e.StudentGroupId).HasColumnName("student_group_id");
            entity.Property(e => e.StudentsCount).HasColumnName("students_count");
        });

        OnModelCreatingPartial(modelBuilder);
    }

    partial void OnModelCreatingPartial(ModelBuilder modelBuilder);
}
