﻿using Microsoft.EntityFrameworkCore;
using MsBox.Avalonia.Enums;
using MsBox.Avalonia;
using SocialEducatorAvalonia.Models;
using SocialEducatorAvalonia.ViewModels;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Diagnostics;

namespace SocialEducatorAvalonia.Utilities
{
    public partial class DataEditorUtilities : ViewModelBase
    {
        static string? _selectedFIO = null;
        static string? _selectedGroup = null;
        static string? _selectedStatus = null;

        public static string? SelectedGroup { get => _selectedGroup; set => _selectedGroup = value; }
        public static string? SelectedFIO { get => _selectedFIO; set => _selectedFIO = value; }
        public static string? SelectedStatus { get => _selectedStatus; set => _selectedStatus = value; }

        /// <summary>
        /// Сравнивает поля на наличие значений, и если все не заполнены кроме текущегоЮ то делаем запрос к БД 
        /// </summary>
        /// <returns>Можно ли проводить запрос к бд</returns>
        private static bool CheckAllFields()
        {
            string functionName = new StackTrace().GetFrame(1)?.GetMethod()?.Name ?? "Unknown";

            switch (functionName)
            {
                case "FindByStatus":
                    if (!string.IsNullOrEmpty(SelectedFIO) || !string.IsNullOrEmpty(SelectedGroup))
                        return false;
                    break;

                case "FindByGroup":
                    if (!string.IsNullOrEmpty(SelectedFIO) || !string.IsNullOrEmpty(SelectedStatus))
                        return false;
                    break;

                case "FindUserByFIO":
                    if (!string.IsNullOrEmpty(SelectedGroup) || !string.IsNullOrEmpty(SelectedStatus))
                        return false;
                    break;

                case "FindByPayment":
                    if (!string.IsNullOrEmpty(SelectedGroup) || !string.IsNullOrEmpty(SelectedFIO))
                        return false;
                    break;
            }

            return true;
        }

        /// <summary>
        /// Выбор метода для повторного вызова для обновления данных
        /// </summary>
        /// <param name="value">Параметр по которому будет совершен запрос</param>
        /// <param name="StudentsList">List видимый на странице</param>
        /// <param name="_studentStorage">Collection в запасе</param>
        /// <returns>Новая коллекция</returns>
        private static ObservableCollection<Student>? GoToTheCorrectMethod(string value, ref ObservableCollection<Student> StudentsList, ref List<Student> _studentStorage)
        {
            if (SelectedFIO == value)
                return FindUserByFIO(value, ref StudentsList, ref _studentStorage);
            else
            {
                if (SelectedGroup == value)
                    return FindByGroup(value, ref _studentStorage, ref StudentsList);
                else
                    return FindByStatus(value, ref StudentsList, ref _studentStorage);
            }
        }

        /// <summary>
        /// Проверка всех полей
        /// </summary>
        /// <param name="firstField">первое поле для сравнения</param>
        /// <param name="secondField">второе поле для сравнения</param>
        /// <param name="StudentsList"></param>
        /// <param name="_studentsStorage">Collection в кэше</param>
        /// <returns>Новая коллекция на представление</returns>
        private static ObservableCollection<Student>? CheckFields(string? firstField, string? secondField, ref ObservableCollection<Student> StudentsList, ref List<Student> _studentsStorage)
        {
            if (string.IsNullOrEmpty(firstField) || string.IsNullOrEmpty(secondField)) // if one or two field is empty or null
            {                
                if (string.IsNullOrEmpty(firstField) == string.IsNullOrEmpty(secondField)) // if they are both null or ""
                {            
                    return new ObservableCollection<Student>(DbContext.StudentsActiveAndGraduations.Include(x => x.StudentStatus).Include(x => x.StudentGroup).Take(200).AsEnumerable()
                        .Select(x =>
                        {
                            string[] spilledFio = x.Fio.Split(' ');
                            return new Student
                            {
                                StudentId = x.StudentId,
                                Fio = x.Fio,
                                Surname = spilledFio.Length > 0 ? spilledFio[0] : string.Empty,
                                Name = spilledFio.Length > 1 ? spilledFio[1] : string.Empty,
                                Patronymic = spilledFio.Length > 2 ? spilledFio[2] : string.Empty,
                                AdmissionDate = x.AdmissionDate,
                                StudentGroupId = x.StudentGroupId,
                                StudentStatusId = x.StudentStatusId,
                                StudentGroup = x.StudentGroup,
                                StudentStatus = x.StudentStatus
                            };
                        })); // all values is empty
                }
                else if (!string.IsNullOrEmpty(firstField)) // one of them empty
                    return GoToTheCorrectMethod(firstField, ref StudentsList, ref _studentsStorage);
                else if (!string.IsNullOrEmpty(secondField))
                    return GoToTheCorrectMethod(secondField, ref StudentsList, ref _studentsStorage);

                return null;
            }
            else // two values are not empty
            {                
                var studentList = GoToTheCorrectMethod(firstField, ref StudentsList, ref _studentsStorage);
                var newStudentList = GoToTheCorrectMethod(secondField, ref StudentsList, ref _studentsStorage);

                if (studentList is not null && newStudentList is not null)
                {
                    foreach (Student student in newStudentList)
                    {
                        studentList.Add(student);
                    }
                }
                else
                    return newStudentList;

                return studentList;
            }

        }

        /// <summary>
        /// Поиск по студентам
        /// </summary>
        /// <param name="value">значение по которому ведется поиск</param>
        /// <param name="StudentsList">List отображаемый</param>
        /// <param name="_studentStorage">Collection в кэше</param>
        /// <returns>Новая коллекция на представление</returns>
        public static ObservableCollection<Student>? FindUserByFIO(string? value, ref ObservableCollection<Student> StudentsList, ref List<Student> _studentStorage)
        {
            if (!string.IsNullOrEmpty(value))
            {
                List<Student>? students = new();
                if (CheckAllFields())
                    students = DbContext.Students.FromSql($"select * from search_students_by_fio({value})").ToList(); // можем менять  
                else
                    students = StudentsList.Where(x => x.Fio!.ToLower().Contains(value.ToLower())).ToList(); // не можем 

                return new ObservableCollection<Student>(students);
            }
            else
                return CheckFields(SelectedStatus, SelectedGroup, ref StudentsList, ref _studentStorage); // value is empty
        }

        /// <summary>
        /// Поиск по группам
        /// </summary>
        /// <param name="value">значение по которому ищем</param>
        /// <param name="_studentsStorage">Collection в кэше</param>
        /// <param name="StudentsList">List отображаемый</param>
        /// <returns>Новая коллекция</returns>
        public static ObservableCollection<Student>? FindByGroup(string? value, ref List<Student> _studentsStorage, ref ObservableCollection<Student> StudentsList)
        {
            if (value is not null)
            {
                int groupId;
                GroupDict.TryGetValue(value, out groupId);

                if (CheckAllFields())
                { // мы можем изменять текущую коллекцию запросом
                    int studentCount = DbContext.Database.SqlQuery<int>($"select get_count_students_by_group({groupId})").AsEnumerable().FirstOrDefault();
                    int studentsInStorage = _studentsStorage.Where(x => x.StudentGroupId == groupId).Count();

                    if (studentCount == studentsInStorage)
                        return new ObservableCollection<Student>(_studentsStorage.Where(x => x.StudentGroupId == groupId).ToList()); // в коллекции внутри кеша есть. берем из неё
                    else if (studentCount == 0)
                    {
                        var box = MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "В данной группе нету студентов", ButtonEnum.Ok, Icon.Database);
                        box.ShowAsync();
                        return null;
                    }
                    else
                        return new ObservableCollection<Student>(DbContext.Students.FromSql($"SELECT * FROM get_students_by_group_id({groupId})").ToList()); // нет в коллекции берем из запроса
                }
                else // мы не можем изменять
                { 
                    int studentsCount = StudentsList.Where(x => x.StudentGroupId == groupId).Count();

                    if (studentsCount > 0)
                        return new ObservableCollection<Student>(StudentsList.Where(x => x.StudentGroupId == groupId));
                    else
                    {
                        var box = MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "С данным условием не существует студентов при данной группой", ButtonEnum.Ok, Icon.Database);
                        box.ShowAsync();
                        return null;
                    }
                }
            }
            else
                return CheckFields(SelectedStatus, SelectedFIO, ref StudentsList, ref _studentsStorage); // value is empty
        }

        /// <summary>
        /// Поиск по статусу
        /// </summary>
        /// <param name="value">параметр по которому идет поиск</param>
        /// <param name="StudentsList">коллекция на странице</param>
        /// <param name="_studentsStorage">коллекция студентов</param>
        /// <returns>Коллекцию студентов</returns>
        public static ObservableCollection<Student>? FindByStatus(string? value, ref ObservableCollection<Student> StudentsList, ref List<Student> _studentsStorage)
        {
            if (value is not null)
            {
                int statusId;
                StatusDict.TryGetValue(value, out statusId);

                if (CheckAllFields())
                    return new ObservableCollection<Student>(DbContext.Students.FromSql($"SELECT * FROM get_students_by_status({statusId})").ToList()); // можем изменять текущую коллекцию
                else
                {
                    // не можем изменять текущую коллекцию
                    int studentCount = StudentsList.Where(x => x.StudentStatusId == statusId).Count();

                    if (studentCount > 0)
                        return new ObservableCollection<Student>(StudentsList.Where(x => x.StudentStatusId == statusId).ToList());
                    else
                    {
                        var box = MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "С данным условием не существует студентов с данным статусом", ButtonEnum.Ok, Icon.Database);
                        box.ShowAsync();
                        return null;
                    }
                }
            }
            else
                return CheckFields(SelectedGroup, SelectedFIO, ref StudentsList, ref _studentsStorage); // value is empty
        }               
    }
}
