﻿using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using MsBox.Avalonia.Enums;
using MsBox.Avalonia;
using SocialEducatorAvalonia.DTOs;
using SocialEducatorAvalonia.Models;
using System.Collections.Generic;
using System.Collections.Immutable;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading.Tasks;
using Avalonia.Controls;
using Avalonia.Platform.Storage;
using SocialEducatorAvalonia.Utilities;
using System.Data;
using System.IO;
using ExcelDataReader;

namespace SocialEducatorAvalonia.ViewModels
{
    partial class AddPaymentsViewModel : ViewModelBase
    {
        [ObservableProperty]
        static ObservableCollection<PersonsPayment> _personsPaymentList = [];

        List<PersonsPayment> _paymentStorage = [];

        readonly ImmutableArray<string> _paymentTypeList;
        public ImmutableArray<string> PaymentTypeList { get => _paymentTypeList; }

        readonly ImmutableSortedDictionary<string, int> _paymentDict;
        public ImmutableSortedDictionary<string, int> PaymentDict { get => _paymentDict; }

        readonly ImmutableArray<string> _paymentsType;
        public ImmutableArray<string> PaymentType { get => _paymentsType; }

        [ObservableProperty]
        decimal _payment = 0;

        [ObservableProperty]
        string _selectedPayment = "Не выбрано";

        [ObservableProperty]
        bool _selectedComboBox = false;

        [RelayCommand]
        void CheckBoxClicked()
        {
            foreach (var item in PersonsPaymentList)
            {
                item.IsChecked = SelectedComboBox;
            }
        }

        // TODO: возможно есть баги с полем IsChecked, к примеру если вручную выделить все элементы CheckBox не изменится

        [RelayCommand]
        void ApplyFilters()
        {
            if (SelectedPayment is not "Не выбрано")
            {
                foreach (var item in _personsPaymentList)
                {
                    if (item.IsChecked)
                        item.PaymentType = SelectedPayment;
                }
            }

            if (Payment > 0)
            {
                foreach (var item in _personsPaymentList)
                {
                    if (item.IsChecked)
                        item.Payment = Payment;
                }
            }
        }

        public AddPaymentsViewModel()
        {
            _paymentTypeList = [.. DbContext.TypesOfPayments.Select(x => x.TypeTittle)];

            _paymentDict = DbContext.TypesOfPayments.OrderBy(x => x.TypeTittle)
                .ToImmutableSortedDictionary(x => x.TypeTittle, x => x.TypeOfPaymentId);

            _paymentsType = PaymentDict.Keys.ToImmutableArray().Add("Не выбрано");
        }

        public static void TakeList(List<Student> students)
        {
            if (students is not null && students.Count >= 1)
            {
                _personsPaymentList = new(students.AsEnumerable().Select(
                person => new PersonsPayment
                {
                    StudentId = person.StudentId,
                    Name = person.Name,
                    Surname = person.Surname,
                    Patronymic = person.Patronymic,
                    FIO = person.Fio,
                    DataAdmission = Convert.ToInt32(person.AdmissionDate),
                    Group = person.StudentGroup!.Title,
                    PaymentType = "Академическая стипендия",
                    CurrentPaymentDate = DateTime.Now,
                    Payment = 770
                }).ToList());
            }
        }

        async Task LoadDataToDataBase()
        {
            List<StudentsAndPayment> payments = [];

            foreach (var payment in PersonsPaymentList)
            {
                if (payment.StudentId > 0)
                {
                    if (payment.EndRangeDate.HasValue)
                    {
                        if (payment.EndRangeDate > payment.CurrentPaymentDate)
                        {
                            int monthsApart = ((payment.EndRangeDate.Value.Year - payment.CurrentPaymentDate.Year) * 12) + payment.EndRangeDate.Value.Month - payment.CurrentPaymentDate.Month;

                            DateTime original = new(payment.CurrentPaymentDate.Year, payment.CurrentPaymentDate.Month, 1, 0, 0, 0);

                            monthsApart++;

                            for (int i = 0; i < monthsApart; i++)
                            {
                                var currentPayment = new StudentsAndPayment()
                                {
                                    StudentId = payment.StudentId,
                                    AmountPayment = payment.Payment,
                                    DataOfPayment = DateOnly.FromDateTime(original),
                                    TypePaymentId = PaymentDict[payment.PaymentType]
                                };

                                original = original.AddMonths(1);

                                payments.Add(currentPayment);
                            }
                        }
                        else
                            await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", $"Вы указали конечную дату раньше левой даты\nВ записи {payment.FIO}", ButtonEnum.Ok, Icon.Info).ShowAsync();
                    }
                    else
                    {
                        StudentsAndPayment currentPayment = new()
                        {
                            StudentId = payment.StudentId,
                            AmountPayment = payment.Payment,
                            DataOfPayment = DateOnly.FromDateTime(payment.CurrentPaymentDate),
                            TypePaymentId = PaymentDict[payment.PaymentType]
                        };

                        payments.Add(currentPayment);
                    }
                }
            }

            if (payments.Count == 1)
                DbContext.StudentsAndPayments.Add(payments[0]);
            else
                DbContext.StudentsAndPayments.AddRange(payments);

            DbContext.SaveChanges();

            await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Вы добавили данные в БД", ButtonEnum.Ok, Icon.Success).ShowAsync();
        }

        [RelayCommand]
        async Task SaveListToDataBase()
        {

            if (PersonsPaymentList.SingleOrDefault(x => x.ExistInDB == false) is not null)
            {
                var box = MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Обнаружены студенты которых нету в базе данных. Вы хотите занести данные, которые отмечены как существующие ", ButtonEnum.OkCancel, Icon.Database);
                ButtonResult response = await box.ShowAsync();

                if (response == ButtonResult.Ok)
                    await LoadDataToDataBase();
            }
            else
                await LoadDataToDataBase();
        }

        [RelayCommand]
        async Task AddRecordOfStudent()
        {
            PersonsPayment payment = new()
            {
                FIO = "Введите ФИО",
                Payment = 0,
                PaymentType = PaymentDict.Keys.Last(),
                CurrentPaymentDate = DateTime.Now,
            };

            if (PersonsPaymentList.Count >= 0 && !PersonsPaymentList.Contains(payment))
                PersonsPaymentList.Add(payment);
            else
                await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Вы добавляете один и тот же элемент, " +
                           "пожалуйста измените информацию у {Введите ФИО, Выплата, Тип Выплаты, Дата}", ButtonEnum.Ok, Icon.Error).ShowAsync();
        }

        [RelayCommand]
        void ClearAllFields() => PersonsPaymentList.Clear();

        [RelayCommand]
        void DeleteField(PersonsPayment payment)
        {
            if (PersonsPaymentList.Count > 1)
                PersonsPaymentList.Remove(payment);
            else
                PersonsPaymentList.Clear();
        }

        [RelayCommand]
        async Task OpenFileDialog(Window currentWindow)
        {
            if (currentWindow is not null)
            {
                IReadOnlyList<IStorageFile>? files = await ExcelUtilities.OpenFileDialogAsync(currentWindow);

                if (files is not null)
                    await FileActions(files);
            }
        }

        private async Task FileActions(IReadOnlyList<IStorageFile> files)
        {
            if (files.Count >= 1)
            {
                if (ExcelUtilities.CheckFirstRowPayments(ref files))
                {
                    for (int i = 0; i < files.Count; i++)
                    {
                        var data = new DataSet();

                        using (var stream = File.OpenRead(files[i].Path.LocalPath))
                        {
                            using (var reader = ExcelReaderFactory.CreateReader(stream))
                            {
                                var config = new ExcelDataSetConfiguration
                                {
                                    ConfigureDataTable = _ => new ExcelDataTableConfiguration
                                    {
                                        UseHeaderRow = true
                                    }
                                };

                                data = reader.AsDataSet(config);
                                reader.Close();
                            }
                            stream.Close();
                        }

                        List<PersonsPayment> payments = new();
                        ExcelUtilities.AddPaymentInfo(ref data, ref _paymentStorage);
                    }

                    PersonsPaymentList = new(_paymentStorage.DistinctBy(x => new { x.FIO, x.Group }));

                    foreach (var item in PersonsPaymentList)
                    {
                        var student = DbContext.StudentsActiveAndGraduations.SingleOrDefault(x => x.Fio == item.FIO && x.StudentGroup.Title == item.Group);

                        if (student is not null)
                        {
                            item.StudentId = student.StudentId;
                            item.ExistInDB = true;
                        }

                    }
                }
                else
                    await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Файл уже открыт", ButtonEnum.Ok, Icon.Error).ShowAsync();            }
            else
                await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Файл не выбран", ButtonEnum.Ok, Icon.Error).ShowAsync();
        }


        [RelayCommand]
        void GoBack()
        {
            MainWindowViewModel.DataEditorTab = 2;
            MainVM.ChangeView(MainWindowViewModel.ListItems[0]);
        }
    }
}
