﻿using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using MsBox.Avalonia.Enums;
using MsBox.Avalonia;
using SocialEducatorAvalonia.DTOs;
using SocialEducatorAvalonia.Models;
using System.Linq;
using System.Threading.Tasks;
using NPOI.Util;
using System.Text.RegularExpressions;
using Microsoft.EntityFrameworkCore;
using System.Collections.Immutable;

namespace SocialEducatorAvalonia.ViewModels
{
    internal partial class EditStudentViewModel : ViewModelBase
    {
        #region arrays in combo boxes
        readonly ImmutableArray<string> _groupArray;
        public ImmutableArray<string> GroupArray { get => _groupArray; }

        readonly ImmutableArray<string> _statusArray;
        public ImmutableArray<string> StatusArray { get => _statusArray; }
        #endregion

        /// <summary>
        /// Запись отображаемая на странице
        /// </summary>
        [ObservableProperty]
        Persons _currentStudent = new();

        /// <summary>
        /// Запись которая поступила в класс
        /// </summary>
        Persons _studentInStorage = new();

        /// <summary>
        /// Regular there is only Russian characters
        /// </summary>
        private static readonly Regex _regex = new Regex(@"^[а-яА-яёЁ]+$");

        public EditStudentViewModel(Student currentStudent)
        {
            _groupArray = GroupDict.Keys.ToImmutableArray();
            _statusArray = StatusDict.Keys.ToImmutableArray();

            var student = new Persons()
            {
                Id = currentStudent.StudentId,
                Name = currentStudent.Name,
                Surname = currentStudent.Surname,
                Patronymic = currentStudent.Patronymic,
                FIO = currentStudent.Surname.Trim() + " " + currentStudent.Name.Trim() + " " + currentStudent.Patronymic.Trim(),
                Status = StatusDict.FirstOrDefault(x => x.Value == currentStudent.StudentStatusId).Key,
                Group = GroupDict.FirstOrDefault(x => x.Value == currentStudent.StudentGroupId).Key,
                DataAdmission = currentStudent.AdmissionDate,
            };

            CurrentStudent = student;
            _studentInStorage = CurrentStudent.Copy();
        }

        [RelayCommand]
        async Task GoBack()
        {
            if (!_studentInStorage.Equals(CurrentStudent))
            {
                var box = MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Данные изменены, если вы нажмете \"Ок\", вы перейдете на страницу и данные не будут сохранены.\nВ ином случае вы останетесь на этой странице", ButtonEnum.OkCancel, Icon.Database);
                ButtonResult response = await box.ShowAsync();

                if (response == ButtonResult.Ok)
                {
                    MainWindowViewModel.DataEditorTab = 1;

                    MainVM.ChangeView(MainWindowViewModel.ListItems[0]);
                }
            }
            else
            {
                MainWindowViewModel.DataEditorTab = 1;

                MainVM.ChangeView(MainWindowViewModel.ListItems[0]);
            }
        }

        bool RegularExpressionMatcher(string value)
        {
            if (_regex.IsMatch(value))
                return true;

            return false;
        }


        [RelayCommand]
        async Task UpdateValue()
        {
            if (CurrentStudent.Equals(_studentInStorage))
                await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Данные никак не изменены", ButtonEnum.Ok, Icon.Database).ShowAsync();
            else
            {
                bool[] fio = { RegularExpressionMatcher(CurrentStudent.Name), RegularExpressionMatcher(CurrentStudent.Surname), RegularExpressionMatcher(CurrentStudent.Patronymic) };

                if (fio.Contains(false))
                {
                    // одно из полей заполнено не верно
                    for (int i = 0; i < fio.Length; i++)
                    {
                        if (fio[i] == false)
                        {
                            string value = string.Empty;
                            switch (i)
                            {
                                case 0:
                                    value = "Имя";
                                    break;
                                case 1:
                                    value = "Фамилия";
                                    break;
                                case 2:
                                    value = "Отчество";
                                    break;
                            }

                            var box = MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", $"Поле {value} должно содержать только русские буквы", ButtonEnum.Ok, Icon.Error);
                            await box.ShowAsync();
                        }
                    }
                }
                else
                {
                    var student = await DbContext.Students.SingleOrDefaultAsync(x => x.StudentId == CurrentStudent.Id);

                    if (student is not null)
                    {
                        student.Name = CurrentStudent.Name;
                        student.Surname = CurrentStudent.Surname;
                        student.Patronymic = CurrentStudent.Patronymic;
                        student.Fio = CurrentStudent.Surname + " " + CurrentStudent.Name + " " + CurrentStudent.Patronymic;
                        student.StudentGroupId = GroupDict[CurrentStudent.Group];
                        student.StudentStatusId = StatusDict[CurrentStudent.Status];

                        DbContext.Students.Update(student);
                        await DbContext.SaveChangesAsync();

                        var box = MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Данные сохранены", ButtonEnum.Ok, Icon.Success);
                        await box.ShowAsync();

                        _studentInStorage = CurrentStudent;
                    }
                }
            }
        }

        [RelayCommand]
        void GoToPayments()
        {
            MainWindowViewModel.DataEditorTab = 2;

            var mvm = MainWindowViewModel.ListItems[0];

            MainVM.ChangeView(MainWindowViewModel.ListItems[0]);

            if (mvm.Page is DataEditorViewModel devm)
            {
                if (devm.SelectedBtn.TabContent is PaymentsViewModel pvm)
                {
                    pvm.SelectedGroup = CurrentStudent.Group;
                    pvm.SelectedFIO = CurrentStudent.FIO;
                }
            }
        }

    }
}
