﻿using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using MsBox.Avalonia.Enums;
using MsBox.Avalonia;
using SocialEducatorAvalonia.Models;
using System.Collections.Generic;
using System.Collections.Immutable;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading.Tasks;
using System.Diagnostics;

namespace SocialEducatorAvalonia.ViewModels
{
    /// <summary>
    /// Просмотр выплат по студенту
    /// </summary>
    public partial class PersonalStudentPaymentViewModel : ViewModelBase
    {
        #region list & arrays
        readonly ImmutableArray<string> _paymentsArray;
        public ImmutableArray<string> PaymentArray { get => _paymentsArray; }
        #endregion

        [ObservableProperty]
        string _currentPayment = "Не выбрано";

        [ObservableProperty]
        DateTime? _selectedStartDate = null;

        [ObservableProperty]
        DateTime? _selectedEndDate = null;

        readonly Student _currentStudent;
        public Student CurrentStudent { get => _currentStudent; }

        [ObservableProperty]
        ObservableCollection<StudentsAndPayment> _paymentList = [];

        List<StudentsAndPayment> _paymentStorage = [];

        /// <summary>
        /// Обновление даты
        /// </summary>
        /// <returns>Лист выплат</returns>
        List<StudentsAndPayment>? UpdateForDate()
        {
            List<StudentsAndPayment> payments = [];

            if (SelectedEndDate is not null && SelectedStartDate is not null)
            {
                // both not empty
                DateOnly StartedDate = DateOnly.FromDateTime((DateTime)SelectedStartDate);
                DateOnly EndDate = DateOnly.FromDateTime((DateTime)SelectedEndDate);

                if (StartedDate.CompareTo(EndDate) < 0)
                    payments.AddRange(_paymentStorage.Where(x => x.DataOfPayment.CompareTo(StartedDate) >= 0 && x.DataOfPayment.CompareTo(EndDate) <= 0));
                else
                    MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Левое значение должно быть меньше правого в дате", ButtonEnum.Ok, Icon.Warning).ShowAsync();
            }
            else
            {
                // something is not null
                if (SelectedStartDate is not null)
                {
                    DateOnly StartedDate = DateOnly.FromDateTime((DateTime)SelectedStartDate);
                    payments.AddRange(_paymentStorage.Where(x => x.DataOfPayment.CompareTo(StartedDate) >= 0));
                }

                if (SelectedEndDate is not null)
                {
                    DateOnly EndDate = DateOnly.FromDateTime((DateTime)SelectedEndDate);
                    payments.AddRange(_paymentStorage.Where(x => x.DataOfPayment.CompareTo(EndDate) <= 0));
                }
            }

            return payments;
        }


        /// <summary>
        /// Проверка всех полей
        /// </summary>
        void CheckAllFields()
        {
            if (CurrentPayment == "Не выбрано" && SelectedEndDate is null && SelectedStartDate is null)
                PaymentList = new ObservableCollection<StudentsAndPayment>(_paymentStorage.Take(25));
            else
            {
                List<StudentsAndPayment> payments = [];

                string functionName = new StackTrace().GetFrame(1)?.GetMethod()?.Name ?? "Unknown";

                switch (functionName)
                {
                    case "FindByDate":

                        var list = UpdateForDate();

                        if (list is not null)
                            payments.AddRange(list);

                        if (CurrentPayment != "Не выбрано")
                        {
                            var existWithPayment = payments.Where(x => x.TypePayment.TypeTittle == CurrentPayment).ToList();

                            if (existWithPayment.Count > 0)
                                payments = existWithPayment;
                            else
                                MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "В данном диапазоне нету данных с этим типом выплат", ButtonEnum.Ok, Icon.Database).ShowAsync();
                        }

                        break;

                    case "OnCurrentPaymentChanged":

                        var curList = UpdateForDate();

                        if (curList is not null)
                            payments.AddRange(curList);
                        break;


                    case "Unknown":
                        MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Возникла ошибка", ButtonEnum.Ok, Icon.Error).ShowAsync();
                        break;
                }

                PaymentList = new(payments);
            }
        }


        /// <summary>
        /// Поиск по датам
        /// </summary>
        [RelayCommand]
        void FindByDate()
        {
            if (SelectedStartDate is null && SelectedEndDate is null)
            {
                PaymentList = new(_paymentStorage);
                MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Нет не одной даты, обновление коллекции по умолчанию", ButtonEnum.Ok, Icon.Warning).ShowAsync();
            }
            else
                CheckAllFields();
        }

        public PersonalStudentPaymentViewModel(Student student)
        {
            _currentStudent = student;
            _paymentsArray = DbContext.TypesOfPayments.OrderBy(x => x.TypeTittle).Select(x => x.TypeTittle).ToImmutableArray().Add("Не выбрано");

            _paymentStorage = [.. DbContext.StudentsAndPayments.Where(x => x.StudentId == student.StudentId)];
            PaymentList = new ObservableCollection<StudentsAndPayment>(_paymentStorage.Take(25));
        }

        partial void OnCurrentPaymentChanged(string value)
        {
            var payments = PaymentList.Where(x => x.TypePayment.TypeTittle == value);

            if (value != "Не выбрано")
            {
                if (payments.Count() > 0)
                    PaymentList = new ObservableCollection<StudentsAndPayment>(payments);
                else
                {
                    payments = _paymentStorage.Where(x => x.TypePayment.TypeTittle == value);

                    if (payments.Count() > 0)
                        PaymentList = new ObservableCollection<StudentsAndPayment>(payments);
                    else
                        MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "С данным типом выплаты, нет записей у студента", ButtonEnum.Ok, Icon.Database).ShowAsync();
                }
            }
            else // Check to another fields is not empty
                CheckAllFields();
        }

        [RelayCommand]
        async Task DeletePayment(StudentsAndPayment currentPayment)
        {
            ButtonResult result = await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Вы уверены что хотите удалить запись", ButtonEnum.YesNo, Icon.Warning).ShowAsync();

            if (result == ButtonResult.Yes)
            {
                DbContext.StudentsAndPayments.Remove(currentPayment);
                await DbContext.SaveChangesAsync();
                _paymentStorage.Remove(currentPayment);

                if (PaymentList.Count == 1)
                    PaymentList.Clear();
                else
                    PaymentList.Remove(currentPayment);
            }
        }

        [RelayCommand]
        async Task DeleteListAsync()
        {
            var list = PaymentList.Where(x => x.IsChecked == true).ToList();

            if (PaymentList.Count() > 1)
            {
                ButtonResult result = await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Вы уверены что хотите удалить записи", ButtonEnum.YesNo, Icon.Warning).ShowAsync();

                if (result == ButtonResult.Yes)
                {
                    DbContext.StudentsAndPayments.RemoveRange(list);
                    await DbContext.SaveChangesAsync();

                    _paymentStorage.RemoveAll(item => list.Contains(item));

                    if (PaymentList.Count == 1)
                        PaymentList.Clear();
                    else
                    {
                        foreach (var item in list)
                        {
                            PaymentList.Remove(item);
                        }
                    }
                }
            }
            else
                await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Записи или не выбраны или выбран один элемент", ButtonEnum.Ok, Icon.Warning).ShowAsync();

            list.Clear();
        }

        [RelayCommand]
        void GoBack()
        {
            MainWindowViewModel.DataEditorTab = 2;
            MainVM.ChangeView(MainWindowViewModel.ListItems[0]);
        }
    }
}
